/*
Copyright (C) 2015 NTT DATA Corporation

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
 */
package com.clustercontrol.xcloud.aws.ui.dialogs;

import static com.clustercontrol.xcloud.aws.common.AWSConstants.EPROP_AvailabilityZone;

import java.text.MessageFormat;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import com.clustercontrol.ws.xcloud.ExtendedProperty;
import com.clustercontrol.ws.xcloud.Instance;
import com.clustercontrol.xcloud.aws.common.AWSStringConstants;
import com.clustercontrol.xcloud.common.CloudStringConstants;
import com.clustercontrol.xcloud.model.cloud.IStorage;
import com.clustercontrol.xcloud.ui.dialogs.IDialogConstants;
import com.clustercontrol.xcloud.util.ControlUtil;

public class AttachStorageDialog extends Dialog implements CloudStringConstants, AWSStringConstants {
	protected Instance instance;
	protected String deviceName;
	protected IStorage storage;
	
	protected Text device;
	protected Combo cmbInstances;
	
	protected AWSResourceProvider provider;

	/**
	 * Create the dialog.
	 * @param parentShell
	 */
	public AttachStorageDialog(Shell parentShell, IStorage storage) {
		super(parentShell);
		this.storage = storage;
		this.provider = new AWSResourceProvider(storage.getLocation());
	}

	public Instance getInstance(){
		return instance;
	}

	public String getDeviceName(){
		return deviceName;
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite area = (Composite) super.createDialogArea(parent);
		Composite container = new Composite(area, SWT.NONE);
		container.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
		GridLayout gl_container = new GridLayout(2, false);
		container.setLayout(gl_container);
		GridData gd_container = new GridData(GridData.FILL_BOTH);
		gd_container.widthHint = 614;
		container.setLayoutData(gd_container);
		
		Label lblNewLabel = new Label(container, SWT.RIGHT);
		lblNewLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		lblNewLabel.setText(strManager + strSeparator);
		
		Label lblNewLabel_4 = new Label(container, SWT.BORDER);
		lblNewLabel_4.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		lblNewLabel_4.setText(storage.getLocation().getCloudScope().getCloudScopes().getHinemosManager().getManagerName());
		
		Label lblNewLabel_1 = new Label(container, SWT.RIGHT);
		lblNewLabel_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		lblNewLabel_1.setText(strCloudScope + strSeparator);
		
		Label lblNewLabel_5 = new Label(container, SWT.BORDER);
		lblNewLabel_5.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		lblNewLabel_5.setText(String.format("%s (%s)", storage.getLocation().getCloudScope().getName(), storage.getLocation().getCloudScope().getId()));
		
		Label lblVolumeType = new Label(container, SWT.NONE);
		lblVolumeType.setAlignment(SWT.RIGHT);
		lblVolumeType.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		lblVolumeType.setText(strStorage + strSeparator);
		
		Label lblNewLabel_2 = new Label(container, SWT.BORDER);
		lblNewLabel_2.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		lblNewLabel_2.setText(String.format("%s (%s)", storage.getName(), storage.getId()));
		
		Label lblInstance = new Label(container, SWT.NONE);
		lblInstance.setAlignment(SWT.RIGHT);
		lblInstance.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		lblInstance.setText(strCompute + strSeparator);
		
		cmbInstances = new Combo(container, SWT.READ_ONLY);
		cmbInstances.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				checkFinished();
			}
		});
		cmbInstances.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		String availabilityZone = storage.getExtendedProperty(EPROP_AvailabilityZone);
		List<Instance> instances = provider.getInstances();
		for (Instance instance: instances) {
			for (ExtendedProperty prop: instance.getExtendedProperties()) {
				if (!prop.getName().equals(EPROP_AvailabilityZone))
					continue;
				if (prop.getValue().equals(availabilityZone)) {
					String instanceItem = String.format("%s (%s)", instance.getName(), instance.getId());
					cmbInstances.add(instanceItem);
					cmbInstances.setData(instanceItem, instance);
				}
				break;
			}
		}
		
		if (instance != null) {
			for (String item: cmbInstances.getItems()) {
				Instance instance = (Instance)cmbInstances.getData(item);
				if (instance.equals(instance.getId())) {
					cmbInstances.setText(item);
					break;
				}
			}
		}
		
		Label lblSnapshot = new Label(container, SWT.NONE);
		lblSnapshot.setAlignment(SWT.RIGHT);
		lblSnapshot.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		lblSnapshot.setText(strDevice + strSeparator);
		
		device = new Text(container, SWT.BORDER);
		device.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				checkFinished();
			}
		});
		GridData gd_device = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1);
		gd_device.widthHint = 242;
		device.setLayoutData(gd_device);
		
		if (deviceName != null) {
			device.setText(deviceName);
		}
		
		ControlUtil.setRequired(new Control[]{cmbInstances, device});
		
		getShell().getDisplay().asyncExec(new Runnable() {
			@Override
			public void run() {
				checkFinished();
			}
		});

		return area;
	}
	
	/**
	 * Create contents of the button bar.
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				false);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(427, 265);
	}

	@Override
	protected void okPressed() {
		instance = (Instance)cmbInstances.getData(cmbInstances.getText());
		deviceName = device.getText();
		
		super.okPressed();
	}

	@Override
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(MessageFormat.format(dlgStorageAttach, strAws));
	}
	
	protected void checkFinished() {
		Button okButton = getButton(IDialogConstants.OK_ID);
		if (okButton == null)
			return;
			
		if (cmbInstances != null && !cmbInstances.getText().isEmpty() &&
				device != null && !device.getText().isEmpty()
			) {
			getButton(IDialogConstants.OK_ID).setEnabled(true);
		} else {
			getButton(IDialogConstants.OK_ID).setEnabled(false);
		}
	}
}